//! # Simulation Unit Tests
//!
//! This module contains unit tests for simulation-related functionality,
//! including binary loading and system initialization.

/// Tests for binary loader and kernel setup.
pub mod loader;
